@@ -8,12 +8,25 @@
 #' system of linear equations defined by \code{Par}, \code{DM}, and any other constraints using singular value decomposition. 
 #' This can be helpful for exploring relationships between the natural and working scale parameters when covariates are included, but \code{getParDM}
 #' will not necessarily return ``good'' starting values (i.e., \code{Par0}) for \code{\link{fitHMM}} or \code{\link{MIfitHMM}}. 
-#'
-#' @param data Optional \code{\link{momentuHMMData}} object or a data frame containing the covariate values. 
+#' 
+#' @param data Optional \code{\link{momentuHMMData}} object, \code{\link{momentuHierHMMData}} object, or a data frame containing the covariate values. 
 #' \code{data} must be specified if covariates are included in \code{DM}.
+#' 
+#' If a data frame is provided, then either \code{nbStates} and \code{dist} must be specified (for a regular HMM) or \code{hierStates} and \code{hierDist}
+#' must be specified (for a hierarchical HMM).
+#' 
+#' @param ... further arguments passed to or from other methods
+#' @export
+getParDM <- function(data, ...) {
+  UseMethod("getParDM")
+}
+
+#' @rdname getParDM
+#' @method getParDM default
 #' @param nbStates Number of states of the HMM.
 #' @param dist A named list indicating the probability distributions of the data streams. Currently
-#' supported distributions are 'gamma','weibull','exp','lnorm','beta','pois','wrpcauchy', and 'vm'. For example,
+#' supported distributions are 'bern', 'beta', 'exp', 'gamma', 'lnorm', 'norm', 'mvnorm2' (bivariate normal distribution), 'mvnorm3' (trivariate normal distribution),
+#' 'pois', 'rw_norm' (normal random walk), 'rw_mvnorm2' (bivariate normal random walk), 'rw_mvnorm3' (trivariate normal random walk), 'vm', 'vmConsensus', 'weibull', and 'wrpcauchy'. For example,
 #' \code{dist=list(step='gamma', angle='vm', dives='pois')} indicates 3 data streams ('step', 'angle', and 'dives')
 #' and their respective probability distributions ('gamma', 'vm', and 'pois').
 #' @param Par A named list containing vectors of state-dependent probability distribution parameters for 
@@ -21,14 +34,14 @@
 #' in the order expected by the pdfs of \code{dist}, and any zero-mass parameters should be the last.
 #' @param zeroInflation A named list of logicals indicating whether the probability distributions of the data streams should be zero-inflated. If \code{zeroInflation} is \code{TRUE} 
 #' for a given data stream, then values for the zero-mass parameters should be
-#' included in the corresponding element of \code{Par}. Ignored if \code{data} is a \code{\link{momentuHMMData}} object.
+#' included in the corresponding element of \code{Par}. Ignored if \code{data} is a \code{\link{momentuHMMData}} or \code{\link{momentuHierHMMData}} object.
 #' @param oneInflation Named list of logicals indicating whether the probability distributions of the data streams are one-inflated. If \code{oneInflation} is \code{TRUE} 
 #' for a given data stream, then values for the one-mass parameters should be
-#' included in the corresponding element of \code{Par}. Ignored if \code{data} is a \code{\link{momentuHMMData}} object.
+#' included in the corresponding element of \code{Par}. Ignored if \code{data} is a \code{\link{momentuHMMData}} or \code{\link{momentuHierHMMData}} object.
 #' @param estAngleMean An optional named list indicating whether or not to estimate the angle mean for data streams with angular 
 #' distributions ('vm' and 'wrpcauchy'). Any \code{estAngleMean} elements corresponding to data streams that do not have angular distributions are ignored.
-#' @param circularAngleMean An optional named list indicating whether to use circular-linear (FALSE) or circular-circular (TRUE) 
-#' regression on the mean of circular distributions ('vm' and 'wrpcauchy') for turning angles. \code{circularAngleMean} elements corresponding to angular data 
+#' @param circularAngleMean An optional named list indicating whether to use circular-linear or circular-circular 
+#' regression on the mean of circular distributions ('vm' and 'wrpcauchy') for turning angles.  See \code{\link{fitHMM}}. \code{circularAngleMean} elements corresponding to angular data 
 #' streams are ignored unless the corresponding element of \code{estAngleMean} is \code{TRUE}. Any \code{circularAngleMean} elements 
 #' corresponding to data streams that do not have angular distributions are ignored.
 #' @param DM A named list indicating the design matrices to be used for the probability distribution parameters of each data 
@@ -50,7 +63,7 @@
 #' @param workBounds An optional named list of 2-column matrices specifying bounds on the working scale of the probability distribution, transition probability, and initial distribution parameters. For each matrix, the first column pertains to the lower bound and the second column the upper bound.
 #' For data streams, each element of \code{workBounds} should be a k x 2 matrix with the same name of the corresponding element of 
 #' \code{Par0}, where k is the number of parameters. For transition probability parameters, the corresponding element of \code{workBounds} must be a k x 2 matrix named ``beta'', where k=\code{length(beta0)}. For initial distribution parameters, the corresponding element of \code{workBounds} must be a k x 2 matrix named ``delta'', where k=\code{length(delta0)}.
-#' @param workcons An optional named list of vectors specifying constants to add to the regression coefficients on the working scale for 
+#' @param workcons Deprecated: please use \code{workBounds} instead. An optional named list of vectors specifying constants to add to the regression coefficients on the working scale for 
 #' each data stream. Warning: use of \code{workcons} is recommended only for advanced users implementing unusual parameter constraints 
 #' through a combination of \code{DM}, \code{cons}, and \code{workcons}. \code{workcons} is ignored for any given data stream unless \code{DM} is specified.
 #'
@@ -71,16 +84,18 @@
 #' stepDM <- matrix(c(1,1,0,0,0,1,0,0,0,0,1,0,0,0,0,1),4,4,
 #'           dimnames=list(NULL,c("mean:(Intercept)","mean_2",
 #'                                "sd_1:(Intercept)","sd_2:(Intercept)")))
-#' stepcons <- c(1,2,1,1) # coefficient for 'mean_2' constrained to be positive
+#' stepworkBounds <- matrix(c(-Inf,Inf),4,2,byrow=TRUE,
+#'                          dimnames=list(colnames(stepDM),c("lower","upper")))
+#' stepworkBounds["mean_2","lower"] <- 0 #coefficient for 'mean_2' constrained to be positive
 #' wPar0 <- getParDM(nbStates=2,dist=list(step=stepDist),
 #'                       Par=list(step=stepPar0),
-#'                       DM=list(step=stepDM),cons=list(step=stepcons))
+#'                       DM=list(step=stepDM),workBounds=list(step=stepworkBounds))
 #'
 #' \dontrun{
 #' # Fit HMM using wPar0 as initial values for the step data stream
 #' mPar <- fitHMM(data,nbStates=2,dist=list(step=stepDist,angle=angleDist),
 #'                Par0=list(step=wPar0$step,angle=anglePar0),
-#'                DM=list(step=stepDM),cons=list(step=stepcons))
+#'                DM=list(step=stepDM),workBounds=list(step=stepworkBounds))
 #' }
 #' 
 #' # get working parameters for 'DM' using 'cov1' and 'cov2' covariates
@@ -98,24 +113,37 @@
 #'
 #' @importFrom CircStats circ.mean
 #' @importFrom nleqslv nleqslv
+#' @importFrom stats plogis
 #' @export
-getParDM<-function(data=data.frame(),nbStates,dist,
+getParDM.default<-function(data=data.frame(),nbStates,dist,
                  Par,
                  zeroInflation=NULL,
                  oneInflation=NULL,
                  estAngleMean=NULL,
                  circularAngleMean=NULL,
-                 DM=NULL,cons=NULL,userBounds=NULL,workBounds=NULL,workcons=NULL){
+                 DM=NULL,cons=NULL,userBounds=NULL,workBounds=NULL,workcons=NULL, ...){
+  
+  hierArgs <- list(...)
+  argNames <- names(hierArgs)[which(names(hierArgs) %in% c("hierStates","hierDist"))]
+
+  ## check that the data is a momentuHMMData object or valid data frame
+  if(!is.momentuHMMData(data)){ 
+    if(missing(nbStates) & missing(dist)){
+      if(all(c("hierStates","hierDist") %in% argNames)){
+        return(getParDM.hierarchical(data,hierStates=hierArgs$hierStates,hierDist=hierArgs$hierDist,Par,zeroInflation,oneInflation,estAngleMean,circularAngleMean,DM,userBounds,workBounds))
+      }
+    }
+    if(!is.data.frame(data)) stop('data must be a data.frame')
+  }
+  if(!missing(nbStates) | !missing(dist)){
+    if(any(c("hierStates","hierDist") %in% argNames))
+      stop("Either nbStates and dist must be specified (for a regular HMM) or hierStates and hierDist must be specified (for a hierarchical HMM)")
+  }
   
   if(!is.null(cons)) warning("cons argument is deprecated in momentuHMM >= 1.4.0.  Please use workBounds instead.")
   if(!is.null(workcons)) warning("workcons argument is deprecated in momentuHMM >= 1.4.0.  Please use workBounds instead.")
   if(!is.null(workBounds) & (!is.null(cons) | !is.null(workcons))) stop("workBounds cannot be specified when using deprecated arguments cons or workcons; either workBounds or both cons and workcons must be NULL")
   
-  ## check that the data is a momentuHMMData object or valid data frame
-  if(!is.momentuHMMData(data))
-    if(!is.data.frame(data)) stop('data must be a data.frame')
-    #else if(ncol(data)<1) stop('data is empty')
-  
   if(nbStates<1) stop('nbStates must be >0')
   
   if(!is.list(dist) | is.null(names(dist))) stop("'dist' must be a named list")
@@ -125,7 +153,19 @@ getParDM<-function(data=data.frame(),nbStates,dist,
   Par <- Par[distnames]
   
   if(is.momentuHMMData(data)){
-    if(any(is.na(match(distnames,names(data))))) stop(paste0(distnames[is.na(match(distnames,names(data)))],collapse=", ")," not found in data")
+    if(any(is.na(match(distnames,names(data))))){
+      tmpdistnames <- distnames
+      for(i in which(is.na(match(distnames,names(data))))){
+        if(dist[[distnames[i]]] %in% mvndists){
+          if(dist[[distnames[i]]] %in% c("mvnorm2","rw_mvnorm2")){
+            tmpdistnames <- c(tmpdistnames[-i],paste0(distnames[i],".x"),paste0(distnames[i],".y"))
+          } else if(dist[[distnames[i]]] %in% c("mvnorm3","rw_mvnorm3")){
+            tmpdistnames <- c(tmpdistnames[-i],paste0(distnames[i],".x"),paste0(distnames[i],".y"),paste0(distnames[i],".z"))          
+          }
+        }
+      }
+      if(any(is.na(match(tmpdistnames,names(data))))) stop(paste0(tmpdistnames[is.na(match(tmpdistnames,names(data)))],collapse=", ")," not found in data")
+    }
     
     zeroInflation <- vector('list',length(distnames))
     names(zeroInflation) <- distnames
@@ -222,7 +262,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
       }
     }
     nc[[i]] <- nc[[i]]>0
-    if(inputs$circularAngleMean[[i]]) {
+    if(!isFALSE(inputs$circularAngleMean[[i]])) {
       meanind[[i]] <- which((apply(fullDM[[i]][1:nbStates,,drop=FALSE],1,function(x) !all(unlist(x)==0))))
       # deal with angular covariates that are exactly zero
       if(length(meanind[[i]])){
@@ -235,7 +275,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
   }
   
   parCount<- lapply(fullDM,ncol)
-  for(i in distnames[unlist(inputs$circularAngleMean)]){
+  for(i in distnames[!unlist(lapply(inputs$circularAngleMean,isFALSE))]){
     parCount[[i]] <- length(unique(gsub("cos","",gsub("sin","",colnames(fullDM[[i]])))))
   }
   parindex <- c(0,cumsum(unlist(parCount))[-length(fullDM)])
@@ -264,7 +304,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
         
         p<-numeric(length(bndInd))
         if(length(ind1)){
-          if(!inputs$circularAngleMean[[i]]) p[ind1] <- (solve(unique(fullDM[[i]])[ind1,ind1],tan(par[ind1]/2))-workcons[[i]][ind1])^(1/cons[[i]][ind1])
+          if(isFALSE(inputs$circularAngleMean[[i]])) p[ind1] <- (solve(unique(fullDM[[i]])[ind1,ind1],tan(par[ind1]/2))-workcons[[i]][ind1])^(1/cons[[i]][ind1])
           else stop("sorry, circular angle mean parameters are not supported by getParDM")
         }
         
@@ -274,7 +314,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
         ind24<-ind2[which(is.infinite(a[ind2]) & is.infinite(b[ind2]))]
         
         if(length(ind21)) p[ind21]<-(solve(unique(fullDM[[i]])[ind21,ind21],log(par[ind21]-a[ind21]))-workcons[[i]][ind21])^(1/cons[[i]][ind21])
-        if(length(ind22)) p[ind22]<-(solve(unique(fullDM[[i]])[ind22,ind22],logit((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]][ind22])^(1/cons[[i]][ind22])
+        if(length(ind22)) p[ind22]<-(solve(unique(fullDM[[i]])[ind22,ind22],stats::qlogis((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]][ind22])^(1/cons[[i]][ind22])
         if(length(ind23)) p[ind23]<-(solve(unique(fullDM[[i]])[ind23,ind23],-log(-par[ind23]+b[ind23]))-workcons[[i]][ind23])^(1/cons[[i]][ind23])
         if(length(ind24)) p[ind24]<-(solve(unique(fullDM[[i]])[ind24,ind24],par[ind24])-workcons[[i]][ind24])^(1/cons[[i]][ind24])
  
@@ -284,12 +324,12 @@ getParDM<-function(data=data.frame(),nbStates,dist,
         bounds<-inputs$p$bounds[[i]]
         if(any(wpar[[i]]<=bounds[,1] | wpar[[i]]>=bounds[,2])) stop('Par$',i,' must be within parameter bounds')
         if(is.list(inputs$DM[[i]])){
-          if(!inputs$circularAngleMean[[i]])
+          if(isFALSE(inputs$circularAngleMean[[i]]))
             gbInd <- getboundInd(fullDM[[i]])
           else
             gbInd <- c(1:nbStates,getboundInd(fullDM[[i]][1:nbStates+nbStates,])+nbStates)
         } else {
-          if(!inputs$circularAngleMean[[i]])
+          if(isFALSE(inputs$circularAngleMean[[i]]))
             gbInd <- getboundInd(inputs$DM[[i]])
           else 
             gbInd <- c(1:nbStates,getboundInd(inputs$DM[[i]][1:nbStates+nbStates,])+nbStates)
@@ -313,7 +353,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
             
             p<-numeric(parCount[[i]])
             
-            if(!inputs$circularAngleMean[[i]]) {
+            if(isFALSE(inputs$circularAngleMean[[i]])) {
               
               meanind<-which((apply(fullDM[[i]][1:nbStates,,drop=FALSE],2,function(x) !all(unlist(x)==0))))
               
@@ -331,7 +371,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
               if(length(ind22)){
                 asvd<-svd(fullDM[[i]][gbInd,,drop=FALSE][ind22,-meanind,drop=FALSE])
                 adiag <- diag(x=1/asvd$d,nrow=ifelse(length(asvd$d)>1,length(asvd$d),1),ncol=ifelse(length(asvd$d)>1,length(asvd$d),1))
-                p[-meanind] <- ((asvd$v %*% adiag %*% t(asvd$u) %*% logit((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]][-meanind])^(1/cons[[i]][-meanind])
+                p[-meanind] <- ((asvd$v %*% adiag %*% t(asvd$u) %*% stats::qlogis((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]][-meanind])^(1/cons[[i]][-meanind])
               }
               
               if(length(ind23)){
@@ -356,13 +396,13 @@ getParDM<-function(data=data.frame(),nbStates,dist,
               #xmat <- fullDM[[i]][gbInd,,drop=FALSE][meanind1,meanind2,drop=FALSE]
               ##nc<-apply(xmat,1:2,function(x) !all(unlist(x)==0))
               
-              solveatan2<-function(x,theta,covs,cons,workcons,nbStates,nc,meanind,oparms){
+              solveatan2<-function(x,theta,covs,cons,workcons,nbStates,nc,meanind,oparms,circularAngleMean){
                 #Xvec <- x^cons+workcons
-                XB<-getXB(covs,1,c(x,rep(1,oparms)),cons,workcons,TRUE,TRUE,FALSE,nbStates,nc,meanind)[meanind,]
+                XB<-getXB(covs,1,c(x,rep(1,oparms)),cons,workcons,TRUE,circularAngleMean,FALSE,nbStates,nc,meanind)[meanind,]
                 c(abs(theta - XB),rep(0,max(0,length(x)-length(theta))))
               }
 
-              if(length(meanind1)) p[1:(length(meanind2)/2)] <- nleqslv::nleqslv(x=rep(1,length(meanind2)/2),fn=solveatan2,theta=par[meanind1],covs=fullDM[[i]],cons=cons[[i]],workcons=workcons[[i]],nbStates=nbStates,nc=nc[[i]],meanind=meanind1,oparms=parCount[[i]]-length(meanind2)/2,control=list(allowSingular=TRUE))$x[1:(length(meanind2)/2)]
+              if(length(meanind1)) p[1:(length(meanind2)/2)] <- nleqslv::nleqslv(x=rep(1,length(meanind2)/2),fn=solveatan2,theta=par[meanind1],covs=fullDM[[i]],cons=cons[[i]],workcons=workcons[[i]],nbStates=nbStates,nc=nc[[i]],meanind=meanind1,oparms=parCount[[i]]-length(meanind2)/2,circularAngleMean=inputs$circularAngleMean[[i]],control=list(allowSingular=TRUE))$x[1:(length(meanind2)/2)]
               
               meanind<-which((apply(fullDM[[i]][nbStates+1:nbStates,,drop=FALSE],2,function(x) !all(unlist(x)==0))))
               
@@ -375,7 +415,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
               if(length(ind22)){
                 asvd<-svd(fullDM[[i]][gbInd,,drop=FALSE][ind22,meanind,drop=FALSE])
                 adiag <- diag(x=1/asvd$d,nrow=ifelse(length(asvd$d)>1,length(asvd$d),1),ncol=ifelse(length(asvd$d)>1,length(asvd$d),1))
-                p[meanind-length(meanind2)/2] <- ((asvd$v %*% adiag %*% t(asvd$u) %*% logit((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]][meanind-length(meanind2)/2])^(1/cons[[i]][meanind-length(meanind2)/2])
+                p[meanind-length(meanind2)/2] <- ((asvd$v %*% adiag %*% t(asvd$u) %*% stats::qlogis((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]][meanind-length(meanind2)/2])^(1/cons[[i]][meanind-length(meanind2)/2])
               }
               
               if(length(ind23)){
@@ -394,7 +434,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
           } else if(!length(ind2)){
             p <- (solve(fullDM[[i]][gbInd,],tan(par/2))-workcons[[i]])^(1/cons[[i]])
           } else stop("sorry, the parameters for ",i," cannot have different bounds")
-        } else if(((length(ind21)>0) + (length(ind22)>0) + (length(ind23)>0))>1){ 
+        } else if(((length(ind21)>0) + (length(ind22)>0) + (length(ind23)>0) + (length(ind24)>0))>1){ 
           stop("sorry, getParDM requires the parameters for ",i," to have identical bounds when covariates are included in the design matrix")
         } else {
           if(length(ind21)){
@@ -406,7 +446,7 @@ getParDM<-function(data=data.frame(),nbStates,dist,
           if(length(ind22)){
             asvd<-svd(fullDM[[i]][gbInd,,drop=FALSE][ind22,,drop=FALSE])
             adiag <- diag(x=1/asvd$d,nrow=ifelse(length(asvd$d)>1,length(asvd$d),1),ncol=ifelse(length(asvd$d)>1,length(asvd$d),1))
-            p <- ((asvd$v %*% adiag %*% t(asvd$u) %*% logit((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]])^(1/cons[[i]])
+            p <- ((asvd$v %*% adiag %*% t(asvd$u) %*% stats::qlogis((par[ind22]-a[ind22])/(b[ind22]-a[ind22])))-workcons[[i]])^(1/cons[[i]])
           }
           
           if(length(ind23)){
@@ -436,3 +476,30 @@ getParDM<-function(data=data.frame(),nbStates,dist,
   }
   wpar
 }
+
+#' @rdname getParDM
+#' @method getParDM hierarchical
+#' @param hierStates A hierarchical model structure \code{\link[data.tree]{Node}} for the states.  See \code{\link{fitHMM}}.
+#' @param hierDist A hierarchical data structure \code{\link[data.tree]{Node}} for the data streams. See \code{\link{fitHMM}}.
+#'
+#' @export
+getParDM.hierarchical<-function(data=data.frame(), hierStates, hierDist,
+                       Par,
+                       zeroInflation=NULL,
+                       oneInflation=NULL,
+                       estAngleMean=NULL,
+                       circularAngleMean=NULL,
+                       DM=NULL, userBounds=NULL, workBounds=NULL, ...){
+  
+  ## check that the data is a momentuHierHMMData object or valid data frame
+  if(!is.momentuHierHMMData(data)){
+    if(!is.data.frame(data)) stop('data must be a data.frame')
+  } else {
+    data <- momentuHMMData(data)
+  }
+  
+  nbStates <- nbHierStates(hierStates)$nbStates
+  dist <- getHierDist(hierDist,data=NULL,checkData=FALSE)
+  
+  return(getParDM.default(data,nbStates,dist,Par,zeroInflation,oneInflation,estAngleMean,circularAngleMean,DM,cons=NULL,userBounds,workBounds,workcons=NULL))
+}
